package chess.client.comm;

import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class HeartBeatClient extends Thread {

    private static final int HEART_BEAT_INTERVAL = 3000;
    
    private int port;
    private String ip;
    private Socket socket;
    private OutputStream os;
    private boolean run;
    private String clientName;

    public HeartBeatClient(String ip, int port, String clientName) {
        try {
            this.port = port;
            this.ip = ip;
            this.clientName = clientName;
            socket = new Socket(ip, port);
            os = socket.getOutputStream();
        } catch (UnknownHostException ex) {
            Logger.getLogger(HeartBeatClient.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(HeartBeatClient.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public void run() {
        try {
            sleep(2000);
            run = true;
            byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_NAME, clientName, clientName, "", 0, 0, 0, 0);
            send(data);
            data = Protocol.createPacket(Protocol.MSG_TYPE_HEART_BEAT, "", "", "", 0, 0, 0, 0);
            while (run) {
                try {
                    send(data);
                    sleep(HEART_BEAT_INTERVAL);
                } catch (InterruptedException ex) {
                    closeConnection();
                }
            }
        } catch (InterruptedException ex) {
            Logger.getLogger(HeartBeatClient.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void send(byte[] data) {
        try {
            os.write(data, 0, data.length);
            os.flush();
        } catch (IOException ex) {

        }
    }

    public void closeConnection() {
        run = false;
        try {
            if (os != null) {
                os.close();
            }
            if (socket != null) {
                socket.close();
            }
        } catch (IOException ex) {

        }
    }
}
