package chess.client.gui;

import chess.client.comm.ChessClient;
import chess.client.comm.Protocol;
import chess.client.gui.UserInterface;
import chess.client.logic.peace.Bishop;
import chess.client.logic.peace.Knight;
import chess.client.logic.peace.Pawn;
import chess.client.logic.peace.Peace;
import chess.client.logic.peace.Queen;
import chess.client.logic.peace.Rook;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Image;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.DefaultCaret;

public class ChessClientGUIView extends FrameView {

    private static final int DISCONNECTED = 0;
    private static final int CONNECTED = 1;
    private int connectionState;
    private ChessClient client;
    private UserInterface ui;
    private boolean isWhite;
    private boolean boardEnabled;
    private int movesTableRow;
    private int movesTableCol;
    private List<JPanel> eatenPeacesGUI;
    private DefaultTableModel table;
    private List<Peace> eatenPeacesLst;
    private String serverIP;

    public ChessClientGUIView(SingleFrameApplication app) {
        super(app);

        this.boardEnabled = true;
        connectionState = DISCONNECTED;
        eatenPeacesGUI = new ArrayList<JPanel>();
        eatenPeacesLst = new ArrayList<Peace>();

        serverIP = "localhost";
        
        initComponents();

        table = new DefaultTableModel();
        tblMoves.setModel(table);
        table.addColumn("White");
        table.addColumn("Black");

        getFrame().setTitle("ChessChat");

        getFrame().addWindowListener(
                new WindowAdapter() {

                    public void windowClosing(WindowEvent e) {
                        if (client != null) {
                            client.closeConnection();
                        }
                    }
                });

        yPanel.setLayout(new GridLayout(8, 1));
        for (int i = 8; i > 0; i--) {
            yPanel.add(new JLabel(String.valueOf(i)));
        }

        xPanel.setLayout(new GridLayout(1, 8));
        for (int i = 65; i < 73; i++) {
            xPanel.add(new JLabel(String.valueOf((char) i)));
        }

        isWhite = true;

        initBoard("white");
        enablePanel(pnlMenu, false);

        DefaultCaret caret = (DefaultCaret) txtConsole.getCaret();
        caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);

        connect();

        pnl.setLayout(new GridLayout(10, 1));

        for (int i = 0; i < 10; i++) {
            JPanel p = new JPanel();
            eatenPeacesGUI.add(p);
            pnl.add(p);
        }

        pnl.invalidate();
        pnl.repaint();

    }

    public void addEatenPeace(Peace p) {
        String res = "/resources/";
        ImageIcon icon = null;
        if (eatenPeacesLst.contains(p)) {

            return;
        } else {
            eatenPeacesLst.add(p);
        }
        if (p instanceof Pawn) {
            if (p.isWhitePeace()) {
                icon = getResizedImage(getClass().getResource(res + "PawnWhite.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(9).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(0).add(new JLabel(icon));
                }
            } else {
                icon = getResizedImage(getClass().getResource(res + "PawnBlack.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(0).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(9).add(new JLabel(icon));
                }
            }
        } else if (p instanceof Knight) {
            if (p.isWhitePeace()) {
                icon = getResizedImage(getClass().getResource(res + "KnightWhite.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(8).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(1).add(new JLabel(icon));
                }
            } else {
                icon = getResizedImage(getClass().getResource(res + "KnightBlack.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(1).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(8).add(new JLabel(icon));
                }
            }
        } else if (p instanceof Bishop) {
            if (p.isWhitePeace()) {
                icon = getResizedImage(getClass().getResource(res + "BishopWhite.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(7).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(2).add(new JLabel(icon));
                }
            } else {
                icon = getResizedImage(getClass().getResource(res + "BishopBlack.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(2).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(7).add(new JLabel(icon));
                }
            }
        } else if (p instanceof Rook) {
            if (p.isWhitePeace()) {
                icon = getResizedImage(getClass().getResource(res + "RookWhite.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(6).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(3).add(new JLabel(icon));
                }
            } else {
                icon = getResizedImage(getClass().getResource(res + "RookBlack.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(3).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(6).add(new JLabel(icon));
                }
            }
        } else if (p instanceof Queen) {
            if (p.isWhitePeace()) {
                icon = getResizedImage(getClass().getResource(res + "QueenWhite.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(5).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(4).add(new JLabel(icon));
                }
            } else {
                icon = getResizedImage(getClass().getResource(res + "QueenBlack.png"));
                if (isWhite) {
                    eatenPeacesGUI.get(4).add(new JLabel(icon));
                } else {
                    eatenPeacesGUI.get(5).add(new JLabel(icon));
                }
            }
        }
    }

    private ImageIcon getResizedImage(URL url) {
        ImageIcon icon = new ImageIcon(url);
        int new_w = 2 * icon.getIconWidth() / 3;
        int new_h = 2 * icon.getIconHeight() / 3;

        Image img = icon.getImage();
        Image newimg = img.getScaledInstance(new_w, new_h, java.awt.Image.SCALE_SMOOTH);
        return new ImageIcon(newimg);
    }

    public void addToClientsList(String clientName) {
        DefaultListModel lm = (DefaultListModel) lstClients.getModel();
        if (!lm.contains(clientName) && !clientName.isEmpty()) {
            lm.addElement(clientName);
        }
    }

    public void emptyClientList() {
        DefaultListModel lm = (DefaultListModel) lstClients.getModel();
        lm.removeAllElements();
    }

    public void console(String msg) {
        if (msg.isEmpty()) {
            return;
        }
        txtConsole.append(msg + "\n");
        txtConsole.setCaretPosition(txtConsole.getDocument().getLength());
    }

    public void addMove(String move) {
        // model.addRow(new Object[]{jTextField1.getText(), jTextField2.getText(),jTextField3.getText(),jTextField4.getText()});
        //tblMoves.getModel().setValueAt(move, movesTableRow, movesTableCol);

        if (movesTableRow == 0 || movesTableCol == 1) {
            table.addRow(new Object[]{"", ""});
        }
        table.setValueAt(move, movesTableRow, movesTableCol);
        if (movesTableCol == 1) {
            movesTableCol = 0;
            movesTableRow++;
        } else {
            movesTableCol++;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        txtClientName = new javax.swing.JTextField();
        btnUpdateName = new javax.swing.JButton();
        pnlMenu = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lstClients = new javax.swing.JList();
        txtMsg = new javax.swing.JTextField();
        btnSend = new javax.swing.JButton();
        btnInvite = new javax.swing.JButton();
        btnResign = new javax.swing.JButton();
        btnOfferDraw = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        tblMoves = new javax.swing.JTable();
        jScrollPane2 = new javax.swing.JScrollPane();
        txtConsole = new javax.swing.JTextArea();
        boardPanel = new javax.swing.JPanel();
        lblConnectionState = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        xPanel = new javax.swing.JPanel();
        yPanel = new javax.swing.JPanel();
        pnl = new javax.swing.JPanel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        menuChangeServerIP = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        jButton1 = new javax.swing.JButton();

        mainPanel.setName("mainPanel"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(chess.client.gui.ChessClientGUIApp.class).getContext().getResourceMap(ChessClientGUIView.class);
        txtClientName.setText(resourceMap.getString("txtClientName.text")); // NOI18N
        txtClientName.setName("txtClientName"); // NOI18N
        txtClientName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtClientNameActionPerformed(evt);
            }
        });

        btnUpdateName.setText(resourceMap.getString("btnUpdateName.text")); // NOI18N
        btnUpdateName.setName("btnUpdateName"); // NOI18N
        btnUpdateName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnUpdateNameActionPerformed(evt);
            }
        });

        pnlMenu.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        pnlMenu.setName("pnlMenu"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        lstClients.setModel(new DefaultListModel());
        lstClients.setName("lstClients"); // NOI18N
        jScrollPane1.setViewportView(lstClients);

        txtMsg.setText(resourceMap.getString("txtMsg.text")); // NOI18N
        txtMsg.setName("txtMsg"); // NOI18N
        txtMsg.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtMsgActionPerformed(evt);
            }
        });

        btnSend.setText(resourceMap.getString("btnSend.text")); // NOI18N
        btnSend.setName("btnSend"); // NOI18N
        btnSend.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSendActionPerformed(evt);
            }
        });

        btnInvite.setText(resourceMap.getString("btnInvite.text")); // NOI18N
        btnInvite.setName("btnInvite"); // NOI18N
        btnInvite.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnInviteActionPerformed(evt);
            }
        });

        btnResign.setText(resourceMap.getString("btnResign.text")); // NOI18N
        btnResign.setName("btnResign"); // NOI18N
        btnResign.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnResignActionPerformed(evt);
            }
        });

        btnOfferDraw.setText(resourceMap.getString("btnOfferDraw.text")); // NOI18N
        btnOfferDraw.setName("btnOfferDraw"); // NOI18N
        btnOfferDraw.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOfferDrawActionPerformed(evt);
            }
        });

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jScrollPane3.setName("jScrollPane3"); // NOI18N

        tblMoves.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "White", "Black"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        tblMoves.setName("tblMoves"); // NOI18N
        jScrollPane3.setViewportView(tblMoves);
        tblMoves.getColumnModel().getColumn(0).setResizable(false);
        tblMoves.getColumnModel().getColumn(1).setResizable(false);

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        txtConsole.setColumns(12);
        txtConsole.setEditable(false);
        txtConsole.setRows(5);
        txtConsole.setFocusable(false);
        txtConsole.setName("txtConsole"); // NOI18N
        jScrollPane2.setViewportView(txtConsole);

        javax.swing.GroupLayout pnlMenuLayout = new javax.swing.GroupLayout(pnlMenu);
        pnlMenu.setLayout(pnlMenuLayout);
        pnlMenuLayout.setHorizontalGroup(
            pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlMenuLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 351, Short.MAX_VALUE)
                    .addComponent(btnInvite, javax.swing.GroupLayout.DEFAULT_SIZE, 351, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, pnlMenuLayout.createSequentialGroup()
                        .addComponent(txtMsg, javax.swing.GroupLayout.DEFAULT_SIZE, 293, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnSend))
                    .addComponent(jLabel1)
                    .addGroup(pnlMenuLayout.createSequentialGroup()
                        .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jScrollPane3, 0, 0, Short.MAX_VALUE)
                            .addComponent(btnResign, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 193, Short.MAX_VALUE)
                            .addComponent(btnOfferDraw, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 193, Short.MAX_VALUE))))
                .addContainerGap())
        );
        pnlMenuLayout.setVerticalGroup(
            pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlMenuLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 156, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnInvite)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnResign)
                    .addComponent(btnOfferDraw))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 222, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 222, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pnlMenuLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnSend)
                    .addComponent(txtMsg, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        boardPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        boardPanel.setName("boardPanel"); // NOI18N

        javax.swing.GroupLayout boardPanelLayout = new javax.swing.GroupLayout(boardPanel);
        boardPanel.setLayout(boardPanelLayout);
        boardPanelLayout.setHorizontalGroup(
            boardPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 498, Short.MAX_VALUE)
        );
        boardPanelLayout.setVerticalGroup(
            boardPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 466, Short.MAX_VALUE)
        );

        lblConnectionState.setText(resourceMap.getString("lblConnectionState.text")); // NOI18N
        lblConnectionState.setName("lblConnectionState"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        xPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        xPanel.setName("xPanel"); // NOI18N

        javax.swing.GroupLayout xPanelLayout = new javax.swing.GroupLayout(xPanel);
        xPanel.setLayout(xPanelLayout);
        xPanelLayout.setHorizontalGroup(
            xPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 498, Short.MAX_VALUE)
        );
        xPanelLayout.setVerticalGroup(
            xPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 26, Short.MAX_VALUE)
        );

        yPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        yPanel.setName("yPanel"); // NOI18N

        javax.swing.GroupLayout yPanelLayout = new javax.swing.GroupLayout(yPanel);
        yPanel.setLayout(yPanelLayout);
        yPanelLayout.setHorizontalGroup(
            yPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 25, Short.MAX_VALUE)
        );
        yPanelLayout.setVerticalGroup(
            yPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 466, Short.MAX_VALUE)
        );

        pnl.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
        pnl.setName("pnl"); // NOI18N

        javax.swing.GroupLayout pnlLayout = new javax.swing.GroupLayout(pnl);
        pnl.setLayout(pnlLayout);
        pnlLayout.setHorizontalGroup(
            pnlLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 66, Short.MAX_VALUE)
        );
        pnlLayout.setVerticalGroup(
            pnlLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 466, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, mainPanelLayout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(txtClientName, javax.swing.GroupLayout.PREFERRED_SIZE, 224, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnUpdateName, javax.swing.GroupLayout.PREFERRED_SIZE, 79, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(4, 4, 4)
                        .addComponent(lblConnectionState))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addComponent(yPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(xPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(boardPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(pnl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
                .addGap(6, 6, 6)
                .addComponent(pnlMenu, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(pnlMenu, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel2)
                            .addComponent(txtClientName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(lblConnectionState)
                            .addComponent(btnUpdateName))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(boardPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(yPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(pnl, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(xPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        menuChangeServerIP.setText(resourceMap.getString("menuChangeServerIP.text")); // NOI18N
        menuChangeServerIP.setName("menuChangeServerIP"); // NOI18N
        menuChangeServerIP.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuChangeServerIPActionPerformed(evt);
            }
        });
        fileMenu.add(menuChangeServerIP);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(chess.client.gui.ChessClientGUIApp.class).getContext().getActionMap(ChessClientGUIView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setName("jButton1"); // NOI18N

        setComponent(mainPanel);
        setMenuBar(menuBar);
    }// </editor-fold>//GEN-END:initComponents
    private void btnSendActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSendActionPerformed
        sendMsg();
    }//GEN-LAST:event_btnSendActionPerformed

    private void sendMsg() {
        if (connectionState == CONNECTED) {
            String msg = txtMsg.getText();
            if (!msg.trim().isEmpty()) {
                console("You: " + txtMsg.getText());
                client.sendMsg(txtMsg.getText());
                txtMsg.setText("");
                if (!boardEnabled) {
                    enableBoard(true);
                }
            }
        } else {
            console("You are not connected");
        }
    }

    private void btnUpdateNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnUpdateNameActionPerformed
        if (txtClientName.getText().isEmpty()) {
            console("Invalid name");
        } else {
            client.sendMyName(txtClientName.getText());
        }
}//GEN-LAST:event_btnUpdateNameActionPerformed

    private void connect() {
        if (connectionState == DISCONNECTED) {
            client = new ChessClient(serverIP, Protocol.MAIN_PORT, this);
            client.start();
            connectionState = CONNECTED;
        } else {
            console("Already connected");
        }
    }

    public void lockUpdateName() {
        btnUpdateName.setEnabled(false);
        enablePanel(pnlMenu, true);
    }

    private void btnInviteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnInviteActionPerformed
        String c = (String) lstClients.getSelectedValue();
        if (c != null) {
            client.sendInvite(c);
        }
}//GEN-LAST:event_btnInviteActionPerformed

    private void btnResignActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnResignActionPerformed
        client.resign();
        console("You resigned");
        unlockList();
    }//GEN-LAST:event_btnResignActionPerformed

    private void btnOfferDrawActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOfferDrawActionPerformed
        client.offerDraw();
        console("Draw offer sent");
    }//GEN-LAST:event_btnOfferDrawActionPerformed

    private void txtClientNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtClientNameActionPerformed
        if (txtClientName.getText().isEmpty()) {
            console("Invalid name");
        } else {
            client.sendMyName(txtClientName.getText());
        }
    }//GEN-LAST:event_txtClientNameActionPerformed

    private void txtMsgActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtMsgActionPerformed
        sendMsg();
    }//GEN-LAST:event_txtMsgActionPerformed

    private void menuChangeServerIPActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuChangeServerIPActionPerformed
        String str = JOptionPane.showInputDialog(null, "Server IP Address:", serverIP);
        if (str != null && !str.isEmpty()) {
            JOptionPane.showMessageDialog(null, "Server IP address updated to " + str,
                    "Sucess", 1);
            serverIP = str;
            client.closeConnection();
            connectionState = DISCONNECTED;
            connect();
            if (txtClientName.getText().isEmpty()) {
                console("Invalid name");
            } else {
                client.sendMyName(txtClientName.getText());
            }
        }
}//GEN-LAST:event_menuChangeServerIPActionPerformed

    public void offerDrawQuestion() {
        int answer = JOptionPane.showConfirmDialog(this.getFrame(),
                "Opponent wants to draw. Accept draw offer?");
        if (answer == JOptionPane.YES_OPTION) {
            client.sendDrawReponse("yes");
            unlockList();
        } else if (answer == JOptionPane.NO_OPTION) {
            client.sendDrawReponse("no");
        }

    }

    public void sendNewMove(int x1, int y1, int x2, int y2) {
        String c = (String) lstClients.getSelectedValue();
        if (c != null) {
            client.sendMove(c, x1, y1, x2, y2);
        }

    }

    public void receivedNewMove(int x1, int y1, int x2, int y2) {
        ui.move(x1, y1, x2, y2);
    }

    public void initBoard(String color) {
        System.out.println("INITIATING BORD");

        movesTableRow =
                0;
        movesTableCol =
                0;
        clearMovesTable();

        boardPanel.removeAll();

        if (color.equals("white")) {
            isWhite = true;
        } else {
            isWhite = false;
        }

        ui = new UserInterface(this, isWhite);

        boardPanel.setLayout(new java.awt.BorderLayout());
        boardPanel.add(ui, BorderLayout.CENTER);
        boardPanel.repaint();
        boardPanel.invalidate();

    }

    public void updateConnectionState(String newState) {
        lblConnectionState.setText(newState);
    }

    public void enableBoard(boolean enable) {
        boardEnabled = enable;
        enablePanel(boardPanel, enable);
        System.out.println("Setting board to: " + enable);
        ui.enableMoves(enable);
    }

    private void enablePanel(JPanel pnl, boolean enable) {
        for (Component c : pnl.getComponents()) {
            c.setEnabled(enable);
        }

    }

    public void selectOpponentAndLockList(String oppenent) {
        DefaultListModel lm = (DefaultListModel) lstClients.getModel();
        if (lm.contains(oppenent) && !oppenent.isEmpty()) {
            lstClients.setSelectedValue(oppenent, true);
            lstClients.setEnabled(false);
        }

    }

    public void unlockList() {
        lstClients.setEnabled(true);
    }

    private void clearMovesTable() {
        table = new DefaultTableModel();
        table.addColumn("White");
        table.addColumn("Black");
        tblMoves.setModel(table);
        tblMoves.invalidate();
    /*
    int cols = table.getRowCount();
    for (int i=0; i<cols; i++) {
    table.setValueAt("", i, 0);
    table.setValueAt("", i, 1);
    }
     */
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel boardPanel;
    private javax.swing.JButton btnInvite;
    private javax.swing.JButton btnOfferDraw;
    private javax.swing.JButton btnResign;
    private javax.swing.JButton btnSend;
    private javax.swing.JButton btnUpdateName;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JLabel lblConnectionState;
    private javax.swing.JList lstClients;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem menuChangeServerIP;
    private javax.swing.JPanel pnl;
    private javax.swing.JPanel pnlMenu;
    private javax.swing.JTable tblMoves;
    private javax.swing.JTextField txtClientName;
    private javax.swing.JTextArea txtConsole;
    private javax.swing.JTextField txtMsg;
    private javax.swing.JPanel xPanel;
    private javax.swing.JPanel yPanel;
    // End of variables declaration//GEN-END:variables
}
