package chess.client.gui;

import chess.client.logic.Board;
import chess.client.logic.BoardPosition;
import chess.client.logic.peace.Peace;
import java.awt.event.MouseEvent;
import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

public class UserInterface extends JPanel {

    private static final Color LIGHT = new Color(228, 167, 76);
    private static final Color DARK = new Color(129, 53, 15);
    private static final Color SELECT = new Color(229, 229, 229);
    
    private Board b;
    private List<MyPanel> sqx;
    private Peace currentPeace;
    private boolean white_move;
    private ChessClientGUIView gui;
    private boolean isWhite;
    private boolean canMove;

    public UserInterface(ChessClientGUIView gui, boolean isWhite) {
        this.gui = gui;
        this.setLayout(new GridLayout(8, 8));
        this.isWhite = isWhite;
        b = new Board(isWhite, this);
        canMove = true;
        sqx = new ArrayList<MyPanel>();

        currentPeace = null;
        white_move = true;
        
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                MyPanel p = new MyPanel(j, 7 - i);
                if ((i + j) % 2 == 0) {
                    p.setBackground(LIGHT);
                } else {
                    p.setBackground(DARK);
                }
                sqx.add(p);
                this.add(sqx.get(sqx.size() - 1));
            }
        }
        updateBoardPeaces();
    }

    private void printList(List<BoardPosition> l) {
        for (BoardPosition pos : l) {
            for (int i=0; i<sqx.size(); i++) {
                MyPanel p = sqx.get(i);
                if (p.getXx()==pos.getXpos() && p.getYy()==pos.getYpos()) {
                    sqx.get(i).setBGColor(Color.GREEN);
                }
            }
            System.out.println(pos.getXpos() + "," + pos.getYpos());
        }
    }
    
    private void updateBoardPeaces() {
        for (int i = 0; i < sqx.size(); i++) {
            sqx.get(i).setImage(null);
            int xPos = sqx.get(i).getXx();
            int yPos = sqx.get(i).getYy();
            Peace peace = b.getPeaceAtPos(xPos, yPos);
            if (peace == null) {
                sqx.get(i).setImage(null);
            } else {
                
                String res = "/resources/";
                
                if (peace.getPeaceColor() == Peace.WHITE) {
                    int peaceType = peace.getPeaceType();
                    switch (peaceType) {
                        case Peace.PAWN:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "PawnWhite.png")));
                            break;
                        case Peace.KNIGHT:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "KnightWhite.png")));
                            break;
                        case Peace.BISHOP:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "BishopWhite.png")));
                            break;
                        case Peace.ROOK:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "RookWhite.png")));
                            break;
                        case Peace.QUEEN:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "QueenWhite.png")));
                            break;
                        case Peace.KING:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "KingWhite.png")));
                            break;
                    }
                } else {
                    int peaceType = peace.getPeaceType();
                    switch (peaceType) {
                        case Peace.PAWN:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "PawnBlack.png")));
                            break;
                        case Peace.KNIGHT:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "KnightBlack.png")));
                            break;
                        case Peace.BISHOP:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "BishopBlack.png")));
                            break;
                        case Peace.ROOK:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "RookBlack.png")));
                            break;
                        case Peace.QUEEN:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "QueenBlack.png")));
                            break;
                        case Peace.KING:
                            sqx.get(i).setImage(new ImageIcon(getClass().getResource(res + "KingBlack.png")));
                            break;
                    }
                }
            }
        }
        this.repaint();
    }
    
    public void move(int x1, int y1, int x2, int y2) {
        if (b.moveAsOpponent(x1, y1, x2, y2)) {
            white_move = !white_move;
            updateBoardPeaces();
        }
    }

    public void resetBoard() {
        currentPeace = null;
        white_move = true;
        b = new Board(isWhite, this);
        updateBoardPeaces();
    }
    
    public void console(String msg) {
        gui.console(msg);
    }
    
    public void addMove(String move) {
        gui.addMove(move);
    }
    
    public void addEatenPeace(Peace p) {
        gui.addEatenPeace(p);
    }

    public void initColors() {
        int c = 0;
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                if ((i + j) % 2 == 0) {
                    sqx.get(c).setBackground(LIGHT);
                } else {
                    sqx.get(c).setBackground(DARK);
                }
                c++;
            }
        }        
    }
    
    private int _row = -1;
    private int _col = -1;
    
    public void enableMoves(boolean enable) {
        canMove = enable;
        
    }

    private class MyPanel extends JPanel implements
            MouseMotionListener, MouseListener {

        private int xx,  yy;

        public MyPanel(int xx, int yy) {
            this.xx = xx;
            this.yy = yy;
            addMouseListener(this);
        }

        public void setImage(ImageIcon img) {
            if (img == null) {
                this.removeAll();
            } else {
                this.add(new JLabel(img));
            }
            this.revalidate();
        }
        
        public void setBGColor(Color color) {
            setBackground(color);
        }

        public int getXx() {
            return xx;
        }

        public int getYy() {
            return yy;
        }

        public void mouseDragged(MouseEvent arg0) {

        }

        public void mouseMoved(MouseEvent arg0) {

        }

        public void mouseClicked(MouseEvent arg0) {
            if (!canMove) {
                setBackground(Color.RED);
                return;
            }
            initColors();
            setBackground(SELECT);

            Peace p = b.getPeaceAtPos(getXx(), getYy());
            int tmp_c = getYy();
            int tmp_r = getXx();

            if (currentPeace == null) {
                _col = tmp_c;
                _row = tmp_r;
                if (p != null) {
                    currentPeace = p;
                }
            } else {
                if (white_move) {
                    if (currentPeace.getPeaceColor() == Peace.WHITE) {
                        if (isWhite) {
                            if (b.move(_row, _col, tmp_r, tmp_c)) {
                                white_move = !white_move;
                                updateBoardPeaces();
                                gui.sendNewMove(_row, _col, tmp_r, tmp_c);
                            }
                        }
                    } else {
                        gui.console("White to move");
                    }
                } else {
                    if (!isWhite) {
                        if (currentPeace.getPeaceColor() == Peace.BLACK) {
                            if (b.move(_row, _col, tmp_r, tmp_c)) {
                                white_move = !white_move;
                                updateBoardPeaces();
                                gui.sendNewMove(_row, _col, tmp_r, tmp_c);
                            }
                        } else {
                            gui.console("Black to move");
                        }
                    }
                }

                _row = -1;
                _col = -1;
                currentPeace = null;
            }
        }

        public void mousePressed(MouseEvent arg0) {

        }

        public void mouseReleased(MouseEvent arg0) {

        }

        public void mouseEntered(MouseEvent arg0) {

        }

        public void mouseExited(MouseEvent arg0) {

        }
    }
}
