package chess.client.logic;

import chess.client.gui.UserInterface;
import chess.*;
import chess.client.logic.peace.Bishop;
import chess.client.logic.peace.King;
import chess.client.logic.peace.Knight;
import chess.client.logic.peace.Pawn;
import chess.client.logic.peace.Peace;
import chess.client.logic.peace.Queen;
import chess.client.logic.peace.Rook;
import java.util.List;

public class Board {

    private Peace[][] board;
    private boolean isWhite;
    private boolean isKingAttacked;
    private UserInterface gui;

    public Board(boolean isWhite, UserInterface gui) {
        this.isWhite = isWhite;
        this.gui = gui;
        isKingAttacked = false;
        board = new Peace[8][8];
        initBoard();
    }

    private void initBoard() {
        initWhitePeaces();
        initBlackPeaces();
    }

    private void initWhitePeaces() {

        int y = 0;
        if (isWhite) {
            y = 1;
        } else {
            y = 6;
        }

        // Pawns
        for (int x = 0; x < 8; x++) {
            //int y = 1;
            board[x][y] = new Pawn(Peace.WHITE, x, y, isWhite);
        }

        //Knights
        if (isWhite) {
            board[1][0] = new Knight(Peace.WHITE, 1, 0);
            board[6][0] = new Knight(Peace.WHITE, 6, 0);
        } else {
            board[1][7] = new Knight(Peace.WHITE, 1, 7);
            board[6][7] = new Knight(Peace.WHITE, 6, 7);
        }

        // Bishops
        if (isWhite) {
            board[2][0] = new Bishop(Peace.WHITE, 2, 0);
            board[5][0] = new Bishop(Peace.WHITE, 5, 0);
        } else {
            board[2][7] = new Bishop(Peace.WHITE, 2, 7);
            board[5][7] = new Bishop(Peace.WHITE, 5, 7);
        }

        // Rooks
        if (isWhite) {
            board[0][0] = new Rook(Peace.WHITE, 0, 0);
            board[7][0] = new Rook(Peace.WHITE, 7, 0);
        } else {
            board[0][7] = new Rook(Peace.WHITE, 0, 7);
            board[7][7] = new Rook(Peace.WHITE, 7, 7);
        }

        // The queen
        if (isWhite) {
            board[3][0] = new Queen(Peace.WHITE, 3, 0);
        } else {
            board[4][7] = new Queen(Peace.WHITE, 4, 7);
        }

        // The king
        if (isWhite) {
            board[4][0] = new King(Peace.WHITE, 4, 0);
        } else {
            board[3][7] = new King(Peace.WHITE, 3, 7);
        }

    }

    private void initBlackPeaces() {

        int y = 0;
        if (isWhite) {
            y = 6;
        } else {
            y = 1;
        }

        for (int x = 0; x < 8; x++) {
            //int y = 6;
            board[x][y] = new Pawn(Peace.BLACK, x, y, isWhite);
        }

        // Rooks
        if (!isWhite) {
            board[0][0] = new Rook(Peace.BLACK, 0, 0);
            board[7][0] = new Rook(Peace.BLACK, 7, 0);
        } else {
            board[0][7] = new Rook(Peace.BLACK, 0, 7);
            board[7][7] = new Rook(Peace.BLACK, 7, 7);
        }

        //Knights
        if (!isWhite) {
            board[1][0] = new Knight(Peace.BLACK, 1, 0);
            board[6][0] = new Knight(Peace.BLACK, 6, 0);
        } else {
            board[1][7] = new Knight(Peace.BLACK, 1, 7);
            board[6][7] = new Knight(Peace.BLACK, 6, 7);
        }

        // Bishops
        if (!isWhite) {
            board[2][0] = new Bishop(Peace.BLACK, 2, 0);
            board[5][0] = new Bishop(Peace.BLACK, 5, 0);
        } else {
            board[2][7] = new Bishop(Peace.BLACK, 2, 7);
            board[5][7] = new Bishop(Peace.BLACK, 5, 7);
        }

        // The queen
        if (!isWhite) {
            board[4][0] = new Queen(Peace.BLACK, 4, 0);
        } else {
            board[3][7] = new Queen(Peace.BLACK, 3, 7);
        }

        // The king
        if (!isWhite) {
            board[3][0] = new King(Peace.BLACK, 3, 0);
        } else {
            board[4][7] = new King(Peace.BLACK, 4, 7);
        }
    }

    public Peace getPeaceAtPos(int x, int y) {
        return board[x][y];
    }

    public void printBoard() {
        for (int y = 7; y >= 0; y--) {
            for (int x = 0; x < 8; x++) {
                Peace p = board[x][y];
                if (p == null) {
                    System.out.print(" x ");
                } else {
                    System.out.print(" " + p.getShortPeaceName() + " ");
                }
            }
            System.out.println();
        }
    }

    public boolean moveAsOpponent(int x1, int y1, int x2, int y2) {
        return move(7-x1, 7-y1, 7-x2, 7-y2);
    }

    public boolean move(int x1, int y1, int x2, int y2) {
        Peace p1 = board[x1][y1];
        Peace p2 = board[x2][y2];

        if (p1 == null) {
            System.out.println("No peace at (" + x1 + "," + y1 + ")");
            return false;
        }

        if (p2 != null) {
            if (p1.isWhitePeace() && p2.isWhitePeace() ||
                    p1.isBlackPeace() && p2.isBlackPeace()) {
                System.out.println("Cannot kill same color");
                return false;
            }
        }
        
        String from = getPositionName(x1, y1);
        String to = getPositionName(x2, y2);

        // This covers the case when a pawn wants to move diagonally.
        // That is valid if the pawn wants to kill
        if (p1 instanceof Pawn) {
            if (p2 != null) {
                if (isWhite) { // player is playing as white
                    if (p1.isWhitePeace() && p2.isBlackPeace()) {
                        if (x1 + 1 == x2 && y1 + 1 == y2) { // eat to the right
                            p1 = new Pawn(Peace.WHITE, x1 + 1, y1, isWhite);
                        } else if (x1 - 1 == x2 && y1 + 1 == y2) { // eat to the left
                            p1 = new Pawn(Peace.WHITE, x1 - 1, y1, isWhite);
                        }
                    } else if (p1.isBlackPeace() && p2.isWhitePeace()) {
                        if (x1 + 1 == x2 && y1 - 1 == y2) { // eat to the right
                            p1 = new Pawn(Peace.BLACK, x1 + 1, y1, isWhite);
                        } else if (x1 - 1 == x2 && y1 - 1 == y2) { // eat to the left
                            p1 = new Pawn(Peace.BLACK, x1 - 1, y1, isWhite);
                        }
                    }
                } else { // player is playing as black
                    if (p1.isBlackPeace() && p2.isWhitePeace()) {
                        if (x1 + 1 == x2 && y1 + 1 == y2) { // eat to the right
                            p1 = new Pawn(Peace.BLACK, x1 + 1, y1, isWhite);
                        } else if (x1 - 1 == x2 && y1 + 1 == y2) { // eat to the left
                            p1 = new Pawn(Peace.BLACK, x1 - 1, y1, isWhite);
                        }
                    } else if (p1.isWhitePeace() && p2.isBlackPeace()) {
                        if (x1 + 1 == x2 && y1 - 1 == y2) { // eat to the right
                            p1 = new Pawn(Peace.WHITE, x1 + 1, y1, isWhite);
                        } else if (x1 - 1 == x2 && y1 - 1 == y2) { // eat to the left
                            p1 = new Pawn(Peace.WHITE, x1 - 1, y1, isWhite);
                        }
                    }

                }
            }
        }

        if (p1 instanceof Bishop || p1 instanceof Queen) {
            if (!isDiagonalEmpty(x1, y1, x2, y2)) {
                System.out.println("The " + p1.getPeaceName() + " peace is blocked");
                return false;
            }
        }

        // If is king and is in position to castle
        if (p1 instanceof King && ((King) p1).canCastle()) {
            if (castle(x1, y1, x2, y2)) {
                return true;
            }
        }

        // If rook moved disable casteling
        if (p1 instanceof Rook) {
            if (isWhite) {
                Peace p = null;
                if (p1.isWhitePeace()) {
                    p = board[4][0];
                    if (p != null) {
                        ((King) board[4][0]).setCannotCastle();
                    }
                } else {
                    p = board[4][7];
                    if (p != null) {
                        ((King) board[4][7]).setCannotCastle();
                    }
                }
            } else {
                Peace p = null;
                if (p1.isWhitePeace()) {
                    p = board[3][7];
                    if (p != null) {
                        ((King) board[3][7]).setCannotCastle();
                    }
                } else {
                    p = board[3][0];
                    if (p != null) {
                        ((King) board[3][0]).setCannotCastle();
                    }
                }
            }
        }
        
        boolean res = p1.move(new BoardPosition(x2, y2));
        board[x1][y1] = null;
        board[p1.getPosition().getXpos()][p1.getPosition().getYpos()] = p1;
        
        isKingAttacked = isKingChecked(x2, y2);
        if (isKingAttacked) {
            gui.console("CHECK!!");
        }
        
        if (res) {
            gui.addMove(from + " " + to);
            if (p2 != null) {
                gui.addEatenPeace(p2);
            }
        }
        
        return res;
    }
    
    private boolean castle(int x1, int y1, int x2, int y2) {
        Peace p1 = board[x1][y1];
        if (isWhite) { // is playing as white
            if (x2 == x1 + 2) { // right castle
                p1.setNewPosition(new BoardPosition(x2, y2));
                board[x1][y1] = null;
                board[x2][y2] = p1;
                Peace rook = null;
                if (p1.isWhitePeace()) {
                    rook = board[7][0];
                    board[7][0] = null;
                } else {
                    rook = board[7][7];
                    board[7][7] = null;
                }
                rook.setNewPosition(new BoardPosition(x1 + 1, y1));
                board[rook.getPosition().getXpos()][rook.getPosition().getYpos()] = rook;

                ((King) board[x2][y2]).setCannotCastle();
                return true;
            } else if (x2 == x1 - 2) { // left castle
                p1.setNewPosition(new BoardPosition(x2, y2));
                board[x1][y1] = null;
                board[x2][y2] = p1;
                Peace rook = null;
                if (p1.isWhitePeace()) {
                    rook = board[0][0];
                    board[0][0] = null;
                } else {
                    rook = board[0][7];
                    board[0][7] = null;
                }
                rook.setNewPosition(new BoardPosition(x1 - 1, y1));
                board[rook.getPosition().getXpos()][rook.getPosition().getYpos()] = rook;

                ((King) board[x2][y2]).setCannotCastle();
                return true;
            }
        } else { // is playing as black
            if (x2 == x1 - 2) {
                p1.setNewPosition(new BoardPosition(x2, y2));
                board[x1][y1] = null;
                board[x2][y2] = p1;
                Peace rook = null;
                if (p1.isWhitePeace()) {
                    rook = board[0][7];
                    board[0][7] = null;
                } else {
                    rook = board[0][0];
                    board[0][0] = null;
                }
                rook.setNewPosition(new BoardPosition(x1 - 1, y1));
                board[rook.getPosition().getXpos()][rook.getPosition().getYpos()] = rook;

                ((King) board[x2][y2]).setCannotCastle();
                return true;
            } else if (x2 == x1 + 2) {
                p1.setNewPosition(new BoardPosition(x2, y2));
                board[x1][y1] = null;
                board[x2][y2] = p1;
                Peace rook = null;
                if (p1.isWhitePeace()) {
                    rook = board[7][7];
                    board[7][7] = null;
                } else {
                    rook = board[7][0];
                    board[7][0] = null;
                }
                rook.setNewPosition(new BoardPosition(x1 + 1, y1));
                board[rook.getPosition().getXpos()][rook.getPosition().getYpos()] = rook;

                ((King) board[x2][y2]).setCannotCastle();
                return true;
            }
        }
        return false;
    }

    public boolean isOnDiagonal(int x1, int y1, int x2, int y2) {
        for (int i = 0; i < 8; i++) {
            if (x1 == x2 + i && y1 == y2 + i) {
                return true;
            }
        }
        for (int i = 0; i < 8; i++) {
            if (x1 == x2 - i && y1 == y2 - i) {
                return true;
            }
        }
        for (int i = 0; i < 8; i++) {
            if (x1 == x2 + i && y1 == y2 - i) {
                return true;
            }
        }
        for (int i = 0; i < 8; i++) {
            if (x1 == x2 - i && y1 == y2 + i) {
                return true;
            }
        }
        return false;
    }

    private boolean isDiagonalEmpty(int x1, int y1, int x2, int y2) {
        if (x2 > x1) { // right side
            if (y2 > y1) { // right upper diagonal
                for (int x = x1 + 1; x < x2; x++) {
                    y1 += 1;
                    if (board[x][y1] != null) {
                        return false;
                    }
                }
            } else { //right lower diagonal
                for (int x = x1 + 1; x < x2; x++) {
                    y1 -= 1;
                    if (board[x][y1] != null) {
                        return false;
                    }
                }
            }
        } else { // left side
            if (y2 > y1) { // left upper diagonal
                for (int x = x1 - 1; x > x2; x--) {
                    y1 += 1;
                    if (board[x][y1] != null) {
                        return false;
                    }
                }
            } else { //left lower diagonal
                for (int x = x1 - 1; x > x2; x--) {
                    y1 -= 1;
                    if (board[x][y1] != null) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    
    private boolean isKingChecked(int x, int y) {
        Peace p = getPeaceAtPos(x, y);
        if (p != null)
            return check(p.getAttackingPositions(), p.isWhitePeace());
        return false;        
    }
    
    private boolean check(List<BoardPosition> lst, boolean isWhitePeace) {
        for (BoardPosition bp : lst) {
            if (getPeaceAtPos(bp.getXpos(), bp.getYpos()) instanceof King) {
                if (isWhitePeace && getPeaceAtPos(bp.getXpos(), bp.getYpos()).isBlackPeace()) {
                    return true;
                }
                if (!isWhitePeace && getPeaceAtPos(bp.getXpos(), bp.getYpos()).isWhitePeace()) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public String getPositionName(int x, int y) {
        Peace p = getPeaceAtPos(x, y);
        String posName = "";
        if (p!=null)
            posName = p.getShortPeaceName();
        posName += (String.valueOf((char)(97+x))) + "" + (y+1);
        return posName;
    }
    
}
