package chess.server.comm;

public class Protocol {
    
    public static final int MAIN_PORT       = 45000;
    public static final int HEART_BEAT_PORT = 45001;
    
    public static final int MSG_TYPE_ERROR          = 0;
    public static final int MSG_TYPE_MOVE           = 1;
    public static final int MSG_TYPE_MSG            = 2;
    public static final int MSG_TYPE_PAIR           = 3;
    public static final int MSG_TYPE_CLIENT_LIST    = 4;
    public static final int MSG_TYPE_NAME           = 5;
    public static final int MSG_TYPE_COLOR          = 6;
    public static final int MSG_TYPE_INVITE         = 7;
    public static final int MSG_TYPE_RESIGN         = 8;
    public static final int MSG_TYPE_DRAW_OFFER     = 9;
    public static final int MSG_TYPE_MUST_CHAT      = 10;
    public static final int MSG_TYPE_HEART_BEAT     = 11;
    public static final int MSG_TYPE_NAME_OK        = 12;
    
    private static final int MSG_TYPE_BYTES         = 1;
    private static final int FROM_CLIENT_BYTES      = 32;
    private static final int TO_CLIENT_BYTES        = 32;
    private static final int EXTRA_MSG_BYTES        = 128;
    private static final int MOVES_BYTES            = 4;
    
    public static final int PACKET_SIZE = MSG_TYPE_BYTES + FROM_CLIENT_BYTES + TO_CLIENT_BYTES + EXTRA_MSG_BYTES + MOVES_BYTES;
    public static final int CLIENT_LIST_SIZE = 1025;
    
    public static byte[] createPacket(int msg_type_int, String from_client_s, String to_client_s, 
            String extra_msg_s, byte[] moves) {
        
        byte[] data = new byte[PACKET_SIZE];
        
        byte[] msg_type_b = new byte[]{(byte)msg_type_int};
        byte[] from_client_b = from_client_s.getBytes();
        byte[] to_client_b = to_client_s.getBytes();
        byte[] extra_msg_b = extra_msg_s.getBytes();
        
        System.arraycopy(msg_type_b, 0, data, 0, msg_type_b.length);
        System.arraycopy(from_client_b, 0, data, 1, from_client_b.length);
        System.arraycopy(to_client_b, 0, data, 33, to_client_b.length);
        System.arraycopy(extra_msg_b, 0, data, 65, extra_msg_b.length);
        System.arraycopy(moves, 0, data, 193, moves.length);
        
        return data;
    }
    
    public static byte[] createPacket(int msg_type_int, String from_client_s, String to_client_s, 
            String extra_msg_s, int x1, int y1, int x2, int y2) {
        byte[] data = new byte[PACKET_SIZE];
        
        byte[] msg_type_b = new byte[]{(byte)msg_type_int};
        byte[] from_client_b = from_client_s.getBytes();
        byte[] to_client_b = to_client_s.getBytes();
        byte[] extra_msg_b = extra_msg_s.getBytes();
        byte[] moves_b = new byte[]{(byte)x1, (byte)y1, (byte)x2, (byte)y2};
        
        System.arraycopy(msg_type_b, 0, data, 0, msg_type_b.length);
        System.arraycopy(from_client_b, 0, data, 1, from_client_b.length);
        System.arraycopy(to_client_b, 0, data, 33, to_client_b.length);
        System.arraycopy(extra_msg_b, 0, data, 65, extra_msg_b.length);
        System.arraycopy(moves_b, 0, data, 193, moves_b.length);
        
        return data;
    }
    
    public static byte[] createPacketClientList(String[] clientList) {
        
        byte[] data = new byte[CLIENT_LIST_SIZE];
        
        StringBuilder sb = new StringBuilder();
        for (String c : clientList) {
            sb.append(c);
            sb.append("*");
        }
        byte[] list_b = sb.toString().getBytes();
        
        byte[] msg_type_b = new byte[]{MSG_TYPE_CLIENT_LIST};
        System.arraycopy(msg_type_b, 0, data, 0, msg_type_b.length);
        System.arraycopy(list_b, 0, data, 1, list_b.length);
        
        return data;
        
    }
    
    public static String[] readClientList(byte[] data) {
        byte[] list_b = new byte[Protocol.CLIENT_LIST_SIZE-1];
        System.arraycopy(data, 0, list_b, 0, list_b.length);
        String list_s = new String(list_b);
        String list[] = list_s.split("\\*", -1);
        return list;
    }
    
    public static int readMsgType(byte[] data) {
        return data[0];
    }

    public static String readClientFrom(byte[] data) {
        if (!check(data)) return null;
        byte[] client_b = new byte[FROM_CLIENT_BYTES];
        System.arraycopy(data, 0, client_b, 0, FROM_CLIENT_BYTES);
        return new String(client_b).trim();
    }
    
    public static String readClientTo(byte[] data) {
        if (!check(data)) return null;
        byte[] client_b = new byte[TO_CLIENT_BYTES];
        System.arraycopy(data, 32, client_b, 0, TO_CLIENT_BYTES);
        return new String(client_b).trim();
    }
    
    public static String readExtraMessage(byte[] data) {
        if (!check(data)) return null;
        byte[] msg_b = new byte[EXTRA_MSG_BYTES];
        System.arraycopy(data, 64, msg_b, 0, EXTRA_MSG_BYTES);
        return new String(msg_b).trim();
    }
    
    public static byte[] readMoves(byte[] data) {
        if (!check(data)) return null;
        byte[] moves_b = new byte[MOVES_BYTES];
        System.arraycopy(data, 192, moves_b, 0, MOVES_BYTES);
        return moves_b;
    }
    
    private static boolean check(byte[] data) {
        if (data.length != PACKET_SIZE-1) {
            System.out.println("Invalid packet size");
            return false;
        }
        return true;
    }

}
