package chess.server.gui;

import chess.server.comm.ChessServer;
import chess.server.comm.ClientsManager;
import chess.server.comm.HeartBeatServer;
import chess.server.comm.Protocol;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import javax.swing.DefaultListModel;
import javax.swing.text.DefaultCaret;

public class ChessServerGUIView extends FrameView {

    private ClientsManager manager;
    
    public ChessServerGUIView(SingleFrameApplication app) {
        super(app);

        initComponents();
        
        getFrame().setTitle("ChessChat Server");
        
        DefaultCaret caret = (DefaultCaret)txtConsole.getCaret();
        caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
        
        manager = new ClientsManager(this);
        ChessServer mainServer = new ChessServer(Protocol.MAIN_PORT, manager);
        HeartBeatServer heartbeatServer = new HeartBeatServer(Protocol.HEART_BEAT_PORT, manager);
        
        mainServer.start();
        heartbeatServer.start();        

    }

    public void addToClientsList(String clientName) {
        DefaultListModel lm = (DefaultListModel) lstClients.getModel();
        if (!lm.contains(clientName)) {
            lm.addElement(clientName);
        }
    }
    
    public void removeClientFromList(String clientName) {
        DefaultListModel lm = (DefaultListModel) lstClients.getModel();
        if (lm.contains(clientName)) {
            lm.removeElement(clientName);
            console(clientName + " disconnected");
        }
    }

    public void console(String msg) {
        txtConsole.append(msg + "\n");
    }
    
    public int getNrMovesToChat() {
        return Integer.valueOf(txtNtMovesToChat.getText()) * 2;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lstClients = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        txtConsole = new javax.swing.JTextArea();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        txtMsg = new javax.swing.JTextField();
        btnSendToAllClients = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        txtNtMovesToChat = new javax.swing.JTextField();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();

        mainPanel.setName("mainPanel"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        lstClients.setModel(new DefaultListModel());
        lstClients.setName("lstClients"); // NOI18N
        jScrollPane1.setViewportView(lstClients);

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        txtConsole.setColumns(20);
        txtConsole.setEditable(false);
        txtConsole.setRows(5);
        txtConsole.setFocusable(false);
        txtConsole.setName("txtConsole"); // NOI18N
        jScrollPane2.setViewportView(txtConsole);

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(chess.server.gui.ChessServerGUIApp.class).getContext().getResourceMap(ChessServerGUIView.class);
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        txtMsg.setText(resourceMap.getString("txtMsg.text")); // NOI18N
        txtMsg.setName("txtMsg"); // NOI18N
        txtMsg.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtMsgActionPerformed(evt);
            }
        });

        btnSendToAllClients.setText(resourceMap.getString("btnSendToAllClients.text")); // NOI18N
        btnSendToAllClients.setName("btnSendToAllClients"); // NOI18N
        btnSendToAllClients.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSendToAllClientsActionPerformed(evt);
            }
        });

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        txtNtMovesToChat.setText(resourceMap.getString("txtNtMovesToChat.text")); // NOI18N
        txtNtMovesToChat.setName("txtNtMovesToChat"); // NOI18N

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane1, 0, 0, Short.MAX_VALUE)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addComponent(jLabel3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(txtNtMovesToChat, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(jLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addComponent(jLabel2)
                                .addGap(210, 210, 210))
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 267, Short.MAX_VALUE)))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addComponent(txtMsg, javax.swing.GroupLayout.DEFAULT_SIZE, 334, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnSendToAllClients)))
                .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(txtNtMovesToChat, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 242, Short.MAX_VALUE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 242, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnSendToAllClients)
                    .addComponent(txtMsg, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(chess.server.gui.ChessServerGUIApp.class).getContext().getActionMap(ChessServerGUIView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        setComponent(mainPanel);
        setMenuBar(menuBar);
    }// </editor-fold>//GEN-END:initComponents

    private void btnSendToAllClientsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSendToAllClientsActionPerformed
        if (!txtMsg.getText().isEmpty()) {
            sendToAllClients(txtMsg.getText());
            txtMsg.setText("");
        }
    }//GEN-LAST:event_btnSendToAllClientsActionPerformed

    private void txtMsgActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtMsgActionPerformed
        if (!txtMsg.getText().isEmpty()) {
            sendToAllClients(txtMsg.getText());
            txtMsg.setText("");
        }
    }//GEN-LAST:event_txtMsgActionPerformed

    private void sendToAllClients(String msg) {
        manager.sendToAllClients(msg);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSendToAllClients;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JList lstClients;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JTextArea txtConsole;
    private javax.swing.JTextField txtMsg;
    private javax.swing.JTextField txtNtMovesToChat;
    // End of variables declaration//GEN-END:variables

}
